@@ -17,7 +17,7 @@ enum iaction {
 	IA_STRING, IA_PASTE, IA_REDRAW,
 	IA_KEYPAD, IA_DEFAULT, IA_KEY,
 	IA_MACRO, IA_SCRIPT, IA_PEEK,
-	IA_TYPEAHEAD
+	IA_TYPEAHEAD, IA_FT
 };
 extern enum iaction ia_cause;
 
