@@ -25,6 +25,7 @@ extern void AnsiText_action();
 extern void AsciiField_action();
 extern void Ascii_action();
 extern void CloseScript_action();
+extern void ContinueScript_action();
 extern void EbcdicField_action();
 extern void Ebcdic_action();
 extern void Execute_action();
@@ -33,6 +34,7 @@ extern void Expect_action();
 extern void login_macro();
 extern void macros_init();
 extern void macro_command();
+extern void PauseScript_action();
 extern void ps_set();
 extern void Script_action();
 extern void peer_script_init();
