@@ -282,6 +282,8 @@ static XtResource resources[] = {
 	  offset(color_scheme), XtRString, "default" },
 	{ ResFtCommand, ClsFtCommand, XtRString, sizeof(String),
 	  offset(ft_command), XtRString, "ind$file" },
+	{ ResModifiedSelColor, ClsModifiedSelColor, XtRInt, sizeof(int),
+	  offset(modified_sel_color), XtRString, "10" },
 	{ ResMonoCase, ClsMonoCase, XtRBoolean, sizeof(Boolean),
 	  toggle_offset(MONOCASE), XtRString, ResFalse },
 	{ ResAltCursor, ClsAltCursor, XtRBoolean, sizeof(Boolean),
